import logging
import pandas as pd
import csv
import xlrd
from contextlib import closing
from openpyxl import load_workbook
from airflow.models import BaseOperator
from airflow.utils.decorators import apply_defaults
from airflow.exceptions import AirflowException
from airflow.hooks.base_hook import BaseHook

class NovigiExcelToCSVExportOperator(BaseOperator):

    template_fields = ['input_excel']

    def __init__(self, input_excel: str, output_csv: str, sheet_name: str = None, column_range: range = None, row_range: range = None, headers: list = None, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.input_excel = input_excel
            self.output_csv = output_csv
            self.sheet_name = sheet_name
            self.column_range = column_range
            self.row_range = row_range
            self.headers = headers

    def clean(self, value):
            #use this callback to clean
            return value

    def execute(self, context):

            format_01 = ".xlsx"
            format_02 = ".xls"

            if format_01 in self.input_excel:
                wb = load_workbook(filename = self.input_excel)
                if self.sheet_name is None:
                    sheet = wb.active
                else:
                    sheet = wb[self.sheet_name]

                if self.column_range is None:
                    self.column_range = range(1, sheet.max_column + 1)

                if self.row_range is None:
                    self.row_range = range(1, sheet.max_row + 1)

                with open(self.output_csv, "w") as outfile:
                    writer = csv.writer(outfile, quoting=csv.QUOTE_NONNUMERIC)
                    if self.headers is not None:
                        writer.writerow(headers)
                    for i in self.row_range:
                        row = []
                        for j in self.column_range:
                            cell = sheet.cell(row=i, column=j)
                            row.append(cell.value)

                        writer.writerow([self.clean(x) for x in row])
                    outfile.close()

            elif format_02 in self.input_excel:

                data_xls = pd.read_excel(self.input_excel, self.sheet_name, dtype=str, index_col=None)
                data_xls.to_csv(self.output_csv, encoding='utf-8', index=False)

            else:
                raise ValueError('Particular Excel file type is not supprted when it is converting to a csv file! Supported formats are .xls and .xlsx')

            return True



import logging
from csv import reader
from contextlib import closing

from airflow.models import BaseOperator
from airflow.utils.decorators import apply_defaults
from airflow.exceptions import AirflowException
from airflow.hooks.base_hook import BaseHook

class NovigiCSVToDatabaseExportOperator(BaseOperator):

        template_fields = ['data_file']

        ALLOWED_CONN_TYPE = {
                    "jdbc",
                    "mssql",
                    "mysql",
                    "odbc",
                    "oracle",
                    "postgres"
        }

        def __init__(self, table, data_file, conn_id, operation="INSERT INTO", csv_has_headers=True, column_dict = None, database = None, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.conn_id = conn_id
            self.operation = operation
            self.table = table
            self.data_file = data_file
            self.column_dict = column_dict
            self.csv_has_headers = csv_has_headers
            self.database = database
            self.hook = None

        def get_hook(self):
            conn = BaseHook.get_connection(self.conn_id)

            if conn.conn_type not in self.ALLOWED_CONN_TYPE:
                raise AirflowException(
                        "Invalid connection type. Supported connection types: {}".format(list(self.ALLOWED_CONN_TYPE))
                )

            if not self.hook:
                self.hook = conn.get_hook()
                if self.database:
                    self.hook.schema = self.database

            return self.hook

        def form_insert_query(self, operation, table, columns, row):
            if isinstance(columns, dict):
                column_keys = columns.keys()
            else:
                column_keys = columns

            questions = ','.join(['%s' for item in column_keys])
            if not self.csv_has_headers and self.column_dict is None:
                return """{} {} VALUES ({});""".format(operation, table, questions)
            else:
                column_names = ','.join(column_keys)
                return """{} {} ({}) VALUES ({});""".format(operation, table, column_names, questions)

        def execute(self, context):
            hook = self.get_hook()
            hook.set_autocommit(conn=hook.get_conn(), autocommit=False)
            logging.info("Importing CSV data to SQL. CSV: {}".format(self.data_file))
            conn = hook.get_conn()
            with closing(conn.cursor()) as cursor:
                with open(self.data_file,"r") as infile:
                    csv_reader = reader(infile)
                    if not self.column_dict and self.csv_has_headers:
                        self.column_dict = {k: v for v, k in enumerate(next(csv_reader))}
                    if self.csv_has_headers:
                        next(csv_reader, None)
                    for row in csv_reader:
                        sql_statement = self.form_insert_query(self.operation, self.table, self.column_dict, row)
                        if not self.column_dict:
                            cursor.execute(sql_statement, tuple(None if x == '' else str(x) for x in row))
                        else:
                            cursor.execute(sql_statement, tuple(None if x == '' else str(x) for x in list(map(lambda n: row[n], self.column_dict.values()))))

            conn.commit()

            logging.info("Completed Novigi CSV to Database operator. CSV: {}".format(self.data_file))
            return True



# from Novigi_Operators.custom_csv_to_s3_opertor import CustomCSVToS3Operator
# from Novigi_Operators.novigi_csv_to_db import NovigiCSVToDatabaseExportOperator
# from Novigi_Operators.novigi_excel_to_csv import NovigiExcelToCSVExportOperator
# from Novigi_Operators.novigi_json_to_csv import NovigiJsonToCSVExportOperator
# from Novigi_Operators.novigi_xls_to_csv_operator import NovigiXlsToCSVOperator

## This is needed to allow Airflow to pick up specific metadata fields it needs for certain features. We recognize it's a bit unclean to define these in multiple places, but at this point it's the only workaround if you'd like your custom conn type to show up in the Airflow UI.
def get_provider_info():
    return {
        "package-name": "novigi-operators", # Required
        "name": "novigi-operators", # Required
        "description": "Novigi Custom Airflow Operators", # Required
        "versions": ["1.0"] # Required
    }


import setuptools
from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

"""Perform the package Novigi_Operators setup."""
setup(
    name='novigi-operators',
    version="1.0",
    description='Novigi Custom Airflow Operators',
    long_description=long_description,
    long_description_content_type='text/markdown',
    entry_points={
        "apache_airflow_provider": [
            "provider_info=Novigi_Operators.__init__:get_provider_info"
        ]
    },
    license='Apache License 2.0',
    packages=setuptools.find_packages(),  
    install_requires=['apache-airflow>=2.0',
                      'requests',
                      'jsonpath_ng',
                      'pandas'
                      ],
    setup_requires=['setuptools', 'wheel'],
    author='Novigi',
    author_email='prasadi19943@gmail.com',
    url='https://Prasadi1994@bitbucket.org/novigi/novigi_operators.git',
    python_requires='~=2.7',
)